import java.awt.* ;
import java.lang.Math;

class Ball {
    public static int    iradius ;
    public static int    idiam ;

    public static double radius ;
    public static double diam ;
    public static double diamSquared ;
    public static double accel = 1.0 ;
    public static double hr_accel  = 1.0 ;
    public static double vr_accel  = 1.0 ;
    Color c ;
    double x, y ;
    double dir ;
    double vel ;
    double vrot, hrot ;
    int lastx, lasty ;

    public Ball (Color c,  int x, int y) {
	this.c = c  ;
	this.x = (double)x ;
	this.y = (double)y ;
	dir = 0 ;
	vel = 0 ;
	hrot = vrot = 0 ;
	lastx = x ;
	lasty = y ;
    }
    public static void initStatic (double radius) {
	Ball.iradius = (int)radius ;
	Ball.idiam   = (int)radius * 2 ;

	Ball.radius  = radius ;
	Ball.diam = radius * 2 ;
	Ball.diamSquared = Ball.diam * Ball.diam ;
    }
    public static void setMotionParams (double a, double hr, double vr) {
	accel = a ;
	hr_accel  = hr ;
	vr_accel  = vr ;
    }
    public void setParameter (double dir, double vel, double hrot, double vrot) {
	this.dir = dir ;
	this.vel = vel ;
	this.hrot = hrot ;
	this.vrot = vrot ;
    }
    public void draw (Graphics g) {
	int ix = (int)x - iradius ;
	int iy = (int)y - iradius ;

	g.setColor (c) ;
	g.fillOval (ix, iy, idiam, idiam) ;
	lastx = ix ;
	lasty = iy ;
    }
    public void moveAndDraw (Graphics g, Color bgColor) {
	int ix = (int)x - iradius ;
	int iy = (int)y - iradius ;

	if (ix == lastx && iy == lasty) {
	    g.setColor (c) ;
	    g.fillOval (ix, iy, idiam, idiam) ;
	} else {
	    g.setColor (bgColor) ;
	    g.fillOval (lastx, lasty, idiam, idiam) ;
	}
	g.setColor (c) ;
	g.fillOval (ix, iy, idiam, idiam) ;
	lastx = ix ;
	lasty = iy ;
    }
    public void move () {
	double vx =  vel * Math.sin(dir) ;
	double vy = -vel * Math.cos(dir) ;
	x += vx ;
	y += vy ;
    }
    public boolean disAcceleration () {
	if (hrot < 0) {
	    hrot += hr_accel ;
	    if (hrot > 0)
		hrot = 0 ;
	} else if (hrot > 0) {
	    hrot -= hr_accel ;
	    if (hrot < 0)
		hrot = 0 ;
	}
	if (vrot < 0) {
	    vrot += vr_accel ;
	    if (vrot > 0)
		vrot = 0 ;
	} else if (vrot > 0) {
	    vrot -= vr_accel ;
	    if (vrot < 0)
		vrot = 0 ;
	}
	vel -= accel ;
	if (vel < 0) {
	    vel = 0 ;
	    return true ;
	}
	return false ;
    }
    public void Stop () {
	vel = 0.0 ;
	hrot = vrot = 0 ;
    }
    public double wallRotationFactor () {
	double factor = hrot * 0.7 ;
	hrot *= 0.3 ;
	return factor ;
    }
    public double collisionRotationFactor () {
	double factor = vrot * 0.7 ;
	vrot *= 0.3 ;
	return factor ;
    }
    public static double xyDirection (double x, double y) {
	double ax, ay ;

	if (x == 0 && y == 0)
	    return 0 ;
	ax = Math.abs (x) ;
	ay = Math.abs (y) ;
	if (y < 0) {
	    if (ay >= ax) {
		if (x > 0)
		    return Math.atan (ax/ay) ;
		else
		    return Math.PI * 2.0 - Math.atan (ax/ay) ;
	    } else {
		if (x > 0)
		    return Math.PI / 2.0 - Math.atan (ay/ax) ;
		else
		    return Math.PI * 1.5 + Math.atan (ay/ax) ;
	    }
	} else {
	    if (ax >= ay) {
		if (x > 0)
		    return Math.PI / 2. + Math.atan (ay/ax) ;
		else
		    return Math.PI * 1.5 - Math.atan (ay/ax) ;
	    } else {
		if (x > 0)
		    return Math.PI - Math.atan (ax/ay) ;
		else
		    return Math.PI + Math.atan (ax/ay) ;
	    }
	}
    }
    public boolean leftWall () {
	double vx, vy ;
	vx =  vel * Math.sin (dir) ;
	vy = -vel * Math.cos (dir) ;
	if (vx < 0) {
	    vx = -vx ;
	    vy += wallRotationFactor () ;
	    dir = xyDirection (vx, vy) ;
	    return true ;
	}
	return false ;
    }
    public boolean rightWall () {
	double vx, vy ;

	vx =  vel * Math.sin (dir) ;
	vy = -vel * Math.cos (dir) ;
	if (vx > 0) {
	    vx = -vx ;
	    vy -= wallRotationFactor () ;
	    dir = xyDirection (vx, vy) ;
	    return true ;
	}
	return false ;
    }
    public boolean upperWall () {
	double vx, vy ;

	vx =  vel * Math.sin (dir) ;
	vy = -vel * Math.cos (dir) ;
	if (vy < 0) {
	    vy = -vy ;
	    vx -= wallRotationFactor () ;
	    dir = xyDirection (vx, vy) ;
	    return true ;
	}
	return false ;
    }
    public boolean lowerWall () {
	double vx, vy ;
	
	vx =  vel * Math.sin (dir) ;
	vy = -vel * Math.cos (dir) ;
	if (vy > 0) {
	    vy = -vy ;
	    vx += wallRotationFactor () ;
	    dir = xyDirection (vx, vy) ;
	    return true ;
	}
	return false ;
    }
    public Point getCenter () {
	return new Point ((int)x, (int)y) ;
    }
    public String toString () {
	return " DIR = " + dir + " vel = " + vel ;
    }
}

