//
// MotionManager.java: ܡΰưͤδ
// 	JDK 1.1 version
//				ů
//
import Ball;
import java.lang.Math;

class MotionManager {
    public Ball[] balls ;
    public WallInfo wi ;

    public MotionManager (Ball[] balls, WallInfo wi) {
	this.balls = balls ;
	this.wi = wi ;
    }
    public int collisionCheck (int myball) {
	Ball b1, b2 ;
	int numB1, numB2 ;
	int i, j ;
	double dx, dy ;
	int colbits = 0 ;

	numB1 = balls.length - 1 ;
	numB2 = balls.length ;
	for (i = 0 ; i < numB1 ; i++) {
	    b1 = balls[i] ;
	    for (j = i + 1 ; j < numB2 ; j++) {
		b2 = balls[j] ;
		dx = b2.x - b1.x ;
		dy = b2.y - b1.y ;
		if ((dx * dx) + (dy * dy) < Ball.diamSquared
		    && collisionOccured (b1, b2))
		{
		    if (i == myball) {
			colbits |= (1 << j) ;
		    } else if (j == myball) {
			colbits |= (1 << i) ;
		    }
		}
	    }
	}
	return colbits ;
    }
    public int wallCheck () {
	int ncousions = 0 ;
	int i ;

	for (i = 0 ; i < balls.length ; i++) {
	    Ball b = balls[i] ;
	    if (b.x <= wi.cleft && b.leftWall()) {
		ncousions++ ;
	    }
	    if (b.x >= wi.cright && b.rightWall()) {
		ncousions++ ;
	    }
	    if (b.y <= wi.ctop && b.upperWall()) {
		ncousions++ ;
	    }
	    if (b.y >= wi.cbottom && b.lowerWall()) {
		ncousions++ ;
	    }
	}
	return ncousions ;
    }
    public void moveBalls () {
	int i ;
	for (i = 0 ; i < balls.length ; i++)
	    balls[i].move() ;
    }
    public void stopBalls () {
	int i ;
	for (i = 0 ; i < balls.length ; i++)
	    balls[i].setParameter (0.0, 0.0, 0.0, 0.0) ;
    }
    public boolean disAcceleration () {
	int i ;
	int zerocnt = 0 ;

	for (i = 0 ; i < balls.length; i++) {
	    if (balls[i].disAcceleration()) {
		// true ... ball is stopped
		zerocnt++ ;
	    }
	}
	return (zerocnt == balls.length) ;
    }
    boolean collisionOccured (Ball bA, Ball bB) {
	double col_angle ;
	double dir_a, cos_a, sin_a ;
	double dir_b, cos_b, sin_b ;
	double col_vel_a, nocol_vel_a ;
	double col_vel_b, nocol_vel_b ;
	double tmp ;
	double rot_col_a, rot_nocol_a ;
	double rot_col_b, rot_nocol_b ;
	double new_vel_col_a ;
	double new_vel_col_b ;

	//=== Get collision angle ==
	//
	//           : col_angle      .
	// velocity  : |  _____    .
	// vector a  : | /     \.
	//         ^ : |/ball_b \
	//         |_:_|    +    |
	//        / \: V\ .  \  /
	//       /   | . \___|velocity vector b
	//      |    +....... \ ......
	//       \ ball_a/     |
	//        \_____/      V
	//
	col_angle = Ball.xyDirection (bB.x - bA.x, bB.y - bA.y) ;
	//
	//  To simplify calculation, rotate ball position to
	//  adjust collision angle to ZERO, and rotate velocity directions.
	//
	//
	//             +--------- col_vel_b (velocity element for collision)
	//             |__:__
	//      ball_b |  : +----- dir_b (rotated direction of velocity vecotyr)
	//            /|  : v \                          
	// rotated   | |  + ..- - - -> no_col_vel_b
	// velocity   \+->#   /...      (velocity element NOT for collision)
	// vector a<.. \__V__/    ...> rotated velocity vecotor b
	//            ..  ^  \
	//        dir_a ..#<--- col_vel_a (velocity element for collision)
	//         <- - - +.................
	//          ^ \ ball_a/        
	//no_col_vel_a \_____/         
	// (velocity element NOT for collision)
	//
	dir_a = bA.dir - col_angle ;
	if (dir_a < 0.0)
	    dir_a += Math.PI * 2 ;
	dir_b = bB.dir - col_angle ;
	if (dir_b < 0.0)
	    dir_b += Math.PI * 2 ;
	cos_a = -Math.cos(dir_a) ;
	sin_a =  Math.sin(dir_a) ;
	col_vel_a   = bA.vel * cos_a ;
	nocol_vel_a = bA.vel * sin_a ;

	cos_b = -Math.cos(dir_b) ;
	sin_b =  Math.sin(dir_b) ;
	col_vel_b   = bB.vel * cos_b ;
	nocol_vel_b = bB.vel * sin_b ;
	if (col_vel_a >= col_vel_b)
	    return false ;
	//
	// pick up vertical rotation and convert it to the velocity.
	//
	tmp = bA.collisionRotationFactor() ;
	rot_col_a   = tmp * cos_a ;
	rot_nocol_a = tmp * sin_a ;

	tmp = bB.collisionRotationFactor() ;
	rot_col_b   = tmp * cos_b ;
	rot_nocol_b = tmp * sin_b ;
	
	//
	// exchange velocity element of collision direction
	// and add converted velocity element of vertical rotation
	//
	new_vel_col_a  = col_vel_b + rot_col_a ;
	nocol_vel_a   += rot_nocol_a ;
	
	new_vel_col_b  = col_vel_a + rot_col_b ;
	nocol_vel_b   += rot_nocol_b ;
	
	//
	// calc new absolute value of velocity for ball A
	//
	bA.vel = Math.sqrt (new_vel_col_a * new_vel_col_a
			    + nocol_vel_a * nocol_vel_a) ;
	dir_a = Ball.xyDirection (nocol_vel_a, new_vel_col_a) + col_angle ;
	if (dir_a > Math.PI * 2)
	    dir_a -= Math.PI * 2 ;
	bA.dir = dir_a ;

	//
	// calc new absolute value of velocity for ball B
	//
	bB.vel = Math.sqrt (new_vel_col_b * new_vel_col_b
			    + nocol_vel_b * nocol_vel_b) ;
	//
	// calc new directoion of velocity vectory for ball B
	//
	dir_b = Ball.xyDirection (nocol_vel_b, new_vel_col_b) + col_angle ;
	if (dir_b > Math.PI * 2)
	    dir_b -= Math.PI * 2 ;
	bB.dir = dir_b ;
	return true ;
    }
}

